@@ -10,11 +10,13 @@ from pydicom.dataset import Dataset
 from pydicom.encaps import encapsulate
 from pydicom.sr.codedict import codes
 from pydicom.sr.coding import Code
-from pydicom.valuerep import DA, PersonName, TM
+from pydicom.valuerep import DA, DS, PersonName, TM
 from pydicom.uid import (
     ImplicitVRLittleEndian,
     ExplicitVRLittleEndian,
     RLELossless,
+    JPEGBaseline8Bit,
+    JPEG2000Lossless,
 )
 
 from highdicom.base import SOPClass
@@ -70,7 +72,7 @@ class SCImage(SOPClass):
             study_date: Optional[Union[str, datetime.date]] = None,
             study_time: Optional[Union[str, datetime.time]] = None,
             referring_physician_name: Optional[Union[str, PersonName]] = None,
-            pixel_spacing: Optional[Tuple[int, int]] = None,
+            pixel_spacing: Optional[Tuple[float, float]] = None,
             laterality: Optional[Union[str, LateralityValues]] = None,
             patient_orientation: Optional[
                 Union[
@@ -145,7 +147,7 @@ class SCImage(SOPClass):
            Time of study creation
         referring_physician_name: Union[str, pydicom.valuerep.PersonName, None], optional
             Name of the referring physician
-        pixel_spacing: Union[Tuple[int, int], None], optional
+        pixel_spacing: Union[Tuple[float, float], None], optional
             Physical spacing in millimeter between pixels along the row and
             column dimension
         laterality: Union[str, highdicom.LateralityValues, None], optional
@@ -168,8 +170,12 @@ class SCImage(SOPClass):
             `coordinate_system` is ``"SLIDE"``)
         transfer_syntax_uid: str, optional
             UID of transfer syntax that should be used for encoding of
-            data elements. The following lossless compressed transfer syntaxes
-            are supported: RLE Lossless (``"1.2.840.10008.1.2.5"``).
+            data elements. The following compressed transfer syntaxes
+            are supported: RLE Lossless (``"1.2.840.10008.1.2.5"``), JPEG
+            2000 Lossless (``"1.2.840.10008.1.2.4.90"``), JPEG Baseline
+            (``"1.2.840.10008.1.2.4.50"``). Note that JPEG Baseline is a
+            lossy compression method that will lead to a loss of detail in
+            the image.
         **kwargs: Any, optional
             Additional keyword arguments that will be passed to the constructor
             of `highdicom.base.SOPClass`
@@ -179,6 +185,8 @@ class SCImage(SOPClass):
             ImplicitVRLittleEndian,
             ExplicitVRLittleEndian,
             RLELossless,
+            JPEGBaseline8Bit,
+            JPEG2000Lossless,
         }
         if transfer_syntax_uid not in supported_transfer_syntaxes:
             raise ValueError(
@@ -355,7 +363,13 @@ class SCImage(SOPClass):
                 'Pixel array has an unexpected number of dimensions.'
             )
         if pixel_spacing is not None:
-            self.PixelSpacing = pixel_spacing
+            if len(pixel_spacing) != 2:
+                raise ValueError(
+                    'Argument "pixel_spacing" should have length 2.'
+                )
+            self.PixelSpacing = [
+                DS(ps, auto_format=True) for ps in pixel_spacing
+            ]
 
         encoded_frame = encode_frame(
             pixel_array,
@@ -480,7 +494,8 @@ class SCImage(SOPClass):
         transfer_syntax_uid: str, optional
             UID of transfer syntax that should be used for encoding of
             data elements. The following lossless compressed transfer syntaxes
-            are supported: RLE Lossless (``"1.2.840.10008.1.2.5"``).
+            are supported: RLE Lossless (``"1.2.840.10008.1.2.5"``), JPEG 2000
+            Lossless (``"1.2.840.10008.1.2.4.90"``).
         **kwargs: Any, optional
             Additional keyword arguments that will be passed to the constructor
             of `highdicom.base.SOPClass`
